import logging

from django.urls import reverse
from django_common.auth_backends import User

from web.forms import WorkerForm
from web.models import Worker
from web.models.worker_study_role import WORKER_STAFF, ROLE_CHOICES_DOCTOR, WORKER_VOUCHER_PARTNER
from web.tests import create_worker
from web.tests.functions import create_language, create_location, create_availability, format_form_field
from .. import LoggedInTestCase

logger = logging.getLogger(__name__)


class WorkerViewTests(LoggedInTestCase):
    def test_render_workers_list_request(self):
        create_worker()

        response = self.client.get(reverse('web.views.workers'))
        self.assertEqual(response.status_code, 200)

    def test_render_add_worker_request(self):
        response = self.client.get(reverse('web.views.worker_add', kwargs={'worker_type': WORKER_STAFF}))
        self.assertEqual(response.status_code, 200)

    def test_render_add_worker_request_for_voucher_partner(self):
        response = self.client.get(reverse('web.views.worker_add', kwargs={'worker_type': WORKER_VOUCHER_PARTNER}))
        self.assertEqual(response.status_code, 200)

    def test_render_worker_added_request(self):
        self.assertEqual(1, User.objects.all().count())

        language = create_language()
        location = create_location()

        form_data = self.create_add_worker_form_data(language, location)

        response = self.client.post(reverse('web.views.worker_add', kwargs={'worker_type': WORKER_STAFF}),
                                    data=form_data)

        self.assertEqual(response.status_code, 302)

        self.assertEqual(1, Worker.objects.all().count())
        self.assertEqual(2, User.objects.all().count())

    def create_add_worker_form_data(self, language, location):
        form_data = self.get_form_data(Worker())
        form_data["first_name"] = "John"
        form_data["last_name"] = "Doe"
        form_data["phone_number"] = "0123456789"
        form_data["unit"] = "TEST DEPARTMENT"
        form_data["email"] = "john.doe@unknown.domain.com"
        form_data["specialization"] = "tester"
        form_data["login"] = "tester.login"
        form_data["password"] = "123qweasdzxc"
        form_data["password2"] = "123qweasdzxc"
        form_data["languages"] = [language.id]
        form_data["locations"] = [location.id]
        form_data["role"] = ROLE_CHOICES_DOCTOR
        return form_data

    def test_security_in_worker_added_request(self):
        self.client.logout()

        language = create_language()
        location = create_location()
        count = Worker.objects.all().count()

        form_data = self.create_add_worker_form_data(language, location)

        self.client.post(reverse('web.views.worker_add', kwargs={'worker_type': WORKER_STAFF}), data=form_data)

        new_count = Worker.objects.all().count()
        # new user shouldn't be added
        self.assertEqual(count, new_count)

    @staticmethod
    def get_form_data(worker=None):
        form = WorkerForm(instance=worker)
        form_data = {}
        for key, value in form.initial.items():
            form_data[key] = format_form_field(value)
        return form_data

    def test_render_edit_worker_request(self):
        worker = create_worker()

        form_data = self.get_form_data(worker)
        form_data["last_name"] = "XYZ"
        form_data["role"] = ROLE_CHOICES_DOCTOR

        response = self.client.post(reverse('web.views.worker_edit', args=[worker.id]), data=form_data)
        self.assertEqual(response.status_code, 302)

        updated_worker = Worker.objects.get(id=worker.id)
        self.assertEqual(updated_worker.last_name, form_data["last_name"])

    def test_edit_worker(self):
        worker = create_worker()

        response = self.client.get(reverse('web.views.worker_edit', args=[worker.id]))
        self.assertEqual(response.status_code, 200)

    def test_render_add_availability_request(self):
        worker = create_worker()

        response = self.client.get(reverse('web.views.worker_availability_add', args=[worker.id]))
        self.assertEqual(response.status_code, 200)

    def test_render_edit_availability_request(self):
        worker = create_worker()
        availability = create_availability(worker)

        response = self.client.get(reverse('web.views.worker_availability_edit', args=[availability.id]))
        self.assertEqual(response.status_code, 200)

    def test_render_add_holiday_request(self):
        worker = create_worker()

        response = self.client.get(reverse('web.views.worker_holiday_add', args=[worker.id]))
        self.assertEqual(response.status_code, 200)
