# coding=utf-8
import logging

from django.contrib import messages
from django.shortcuts import redirect, get_object_or_404

from . import wrap_response
from ..forms import VisitDetailForm, SubjectAddForm, SubjectEditForm, StudySubjectAddForm, StudySubjectEditForm
from ..models import StudySubject, MailTemplate, Worker, Study
from ..models.constants import GLOBAL_STUDY_ID
from ..models.study_subject_list import SUBJECT_LIST_GENERIC, SUBJECT_LIST_NO_VISIT, SUBJECT_LIST_REQUIRE_CONTACT, \
    SUBJECT_LIST_VOUCHER_EXPIRY, SUBJECT_LIST_CHOICES

logger = logging.getLogger(__name__)


def subject_list(request, subject_list_type):
    context = {
        'list_type': subject_list_type,
        'worker': Worker.get_by_user(request.user),
        'list_description': SUBJECT_LIST_CHOICES[subject_list_type],
    }
    return wrap_response(request, 'subjects/index.html', context)


def subjects(request):
    return subject_list(request, SUBJECT_LIST_GENERIC)


def subject_add(request):
    study = Study.objects.filter(id=GLOBAL_STUDY_ID)[0]
    if request.method == 'POST':
        study_subject_form = StudySubjectAddForm(request.POST, request.FILES, prefix="study_subject", user=request.user,
                                                 study=study)

        subject_form = SubjectAddForm(request.POST, request.FILES, prefix="subject")
        if study_subject_form.is_valid() and subject_form.is_valid():
            subject = subject_form.save()
            study_subject_form.instance.subject_id = subject.id
            study_subject_form.save()
            messages.add_message(request, messages.SUCCESS, 'Subject created')
            return redirect('web.views.subject_edit', id=study_subject_form.instance.id)
        else:
            messages.add_message(request, messages.ERROR, 'Invalid data. Please fix data and try again.')

    else:
        study_subject_form = StudySubjectAddForm(user=request.user, prefix="study_subject", study=study)
        subject_form = SubjectAddForm(prefix="subject")

    return wrap_response(request, 'subjects/add.html',
                         {'study_subject_form': study_subject_form, 'subject_form': subject_form})


def subject_no_visits(request):
    return subject_list(request, SUBJECT_LIST_NO_VISIT)


def subject_voucher_expiry(request):
    return subject_list(request, SUBJECT_LIST_VOUCHER_EXPIRY)


def subject_require_contact(request):
    return subject_list(request, SUBJECT_LIST_REQUIRE_CONTACT)


def subject_edit(request, id):
    study_subject = get_object_or_404(StudySubject, id=id)
    contact_attempts = study_subject.contactattempt_set.order_by('-datetime_when').all()
    was_dead = study_subject.subject.dead
    was_resigned = study_subject.resigned
    if request.method == 'POST':
        study_subject_form = StudySubjectEditForm(request.POST, request.FILES, instance=study_subject,
                                                  was_resigned=was_resigned, prefix="study_subject"
                                                  )
        subject_form = SubjectEditForm(request.POST, request.FILES, instance=study_subject.subject,
                                       was_dead=was_dead, prefix="subject"
                                       )
        if study_subject_form.is_valid() and subject_form.is_valid():
            study_subject_form.save()
            subject_form.save()
            # check if subject was marked as dead or resigned
            if subject_form.cleaned_data['dead'] and not was_dead:
                study_subject.subject.mark_as_dead()
            if study_subject.study.columns.resigned \
                    and study_subject_form.cleaned_data['resigned'] and not was_resigned:
                study_subject.mark_as_resigned()
            messages.success(request, "Modifications saved")
            if '_continue' in request.POST:
                return redirect('web.views.subject_edit', id=study_subject.id)
            return redirect('web.views.subjects')
        else:
            messages.add_message(request, messages.ERROR, 'Invalid data. Please fix data and try again.')
    else:
        study_subject_form = StudySubjectEditForm(instance=study_subject, was_resigned=was_resigned,
                                                  prefix="study_subject")
        subject_form = SubjectEditForm(instance=study_subject.subject, was_dead=was_dead, prefix="subject")

    languages = []
    if study_subject.subject.default_written_communication_language:
        languages.append(study_subject.subject.default_written_communication_language)
    languages.extend(study_subject.subject.languages.all())

    return wrap_response(request, 'subjects/edit.html', {
        'study_subject_form': study_subject_form,
        'subject_form': subject_form,
        'study_subject': study_subject,
        'contact_attempts': contact_attempts,
        'mail_templates': MailTemplate.get_subject_mail_templates(languages)
    })


def subject_visit_details(request, id):
    study_subject_to_be_viewed = get_object_or_404(StudySubject, id=id)
    visits = study_subject_to_be_viewed.visit_set.order_by("-visit_number").all()
    visits_data = []
    allow_add_visit = True
    for visit in visits:
        appointments = visit.appointment_set.all()
        finished = visit.is_finished
        visit_id = visit.id
        visit_title = "Visit " + str(visit.visit_number)
        visit_form = VisitDetailForm(instance=visit)
        visits_data.append((visit_form, appointments, finished, visit_id, visit_title))
        if not visit.is_finished:
            allow_add_visit = False

    return wrap_response(request, 'subjects/visit_details.html',
                         {'display': visits_data, "id": id, "allow_add_visit": allow_add_visit})
